'''
This module contains functions to handle database interactions.
Document stores like mongodb will be preferred choice for DB.
'''
# Import Libraries
from os import environ
from pymongo import MongoClient
from pandas import read_csv
class db_handler():
    connected_db = None
    def __init__(self,db_details) -> None:
        self.mongo_host = db_details['MONGO_HOST']
        self.mongo_port = db_details['MONGO_PORT']
        self.mongo_db_name = db_details['MONGO_DB_NAME']
        self.mongo_user = db_details['MONGO_USER']
        try:
            self.mongo_db_table_name = db_details['MONGO_DB_I18N_TABLE']
        except:
            pass
        self.mongo_password = db_details['MONGO_PASSWORD']
        if db_details['MONGO_HOST'] != "localhost":
            self.mongo_client = MongoClient(f'mongodb://{self.mongo_user}:{self.mongo_password}@{self.mongo_host}:{self.mongo_port}/?ssl=true&ssl_ca_certs=rds-combined-ca-bundle.pem&replicaSet=rs0&readPreference=secondaryPreferred&retryWrites=false')
        else:
            self.mongo_client = MongoClient(self.mongo_host, int(self.mongo_port))
        self.connected_db = self.mongo_client[self.mongo_db_name]
    def csv_to_db(self,table_name,file_path):
        df = read_csv(file_path)
        data = df.to_dict(orient="records")
        col = self.connected_db[table_name]
        col.insert_many(data)
        print("Completed the csv to db task")
    def get_document_for_key(self,table_name,key ,search_value):
        print(f"Finding document {search_value} in {table_name} mongodb collection in {self.mongo_db_name} database.")
        col = self.connected_db[table_name]
        result_value = col.find_one({
            key : search_value
            })
        return result_value
    def set_document_for_key(self, table_name,key,search_value,replace_key,replace_value):
        print(f"updating {replace_key} to {replace_value} in {table_name} mongodb collection in {self.mongo_db_name} database.")
        col = self.connected_db[table_name]
        query = {key : search_value}
        update_data = {'$set':
            {replace_key : replace_value}
        }
        col.update_one( query, update_data)
        print("Completed set_document_for_key task")
    def get_translated_text_for_lang(self, text, lang_code):
        table_name = self.mongo_db_table_name
        col = self.connected_db[table_name]
        search_query = {'$and':[{'texts': {"$eq":text.strip()}},{'lang_code':{"$eq":lang_code}}]}
        result = col.find_one(search_query)
        if result is not None:
            return result['lang_text']
        return result
    def reverse_translate(self, text):
        table_name = self.mongo_db_table_name
        result = self.get_document_for_key(table_name=table_name, key='lang_text', search_value=text.strip())
        if result is not None:
            return result['texts']
        return text
    def insert_data(self,table_name, entry):
        mycol = self.connected_db[table_name]
        mycol.insert_one(entry)
        return


# Handled by kuldeep
from json import loads


def get_context(dispatcher, tracker, domain):
    print('inside_get_context',tracker.sender_id.lower())
    if 'whatsapp' in tracker.sender_id.lower():
        channel = 'whatsapp'
    elif 'anon' in tracker.sender_id.lower():
        channel = 'gip'
    elif 'instagram' in tracker.sender_id.lower():
        channel = 'instagram'
    else:
        # if 'whatsapp' in str(tracker.sender_id) else "gip"
        channel = 'whatsapp'
    entities = tracker.latest_message["entities"]
    dicts = {}
    for item in entities:
        dicts[item["entity"]] = item["value"]
    entity_dict = dicts
    gs_context_dict = tracker.get_slot("gs_context")
    if gs_context_dict is None:
        slots = {}
        slots['language'] = ""
        active_menu = ""
        active_menu_state = []
        menu_handled = False
        live_dict = {}
        past_conversation = ""
    else:
        gs_context_dict = loads(gs_context_dict)
        slots = gs_context_dict["slots"]
        active_menu = gs_context_dict["active_menu"]
        active_menu_state = gs_context_dict["active_menu_state"]
        menu_handled = gs_context_dict["menu_handled"]
        live_dict = gs_context_dict["live_dict"]
        past_conversation = gs_context_dict["past_conversation"]

    gs_context = {"sender_id": tracker.sender_id, "channel": channel,
                  "active_menu": active_menu, "active_menu_state": active_menu_state, "slots": slots, "menu_handled": menu_handled, "live_dict": live_dict, "past_conversation": past_conversation}
    current_intent = {"intent": tracker.latest_message['intent'].get('name'),
                      "entities": entity_dict,
                      "user_text": tracker.latest_message["text"]}
    current_context = {"current_intent": current_intent,
                       "domain": domain, "response_queue": []}

    return gs_context, current_context



from json import loads
from .i18n import i18n_translate
from . import gs_global


def show_response_card(gs_context,current_context,response_details):
    channel = gs_context['channel']
    lang_code = gs_context["slots"]['language']
    i18n_required = gs_global.global_context['i18n_required']
    if(i18n_required):
        response_details = i18n_translate(response_details,lang_code,channel)
    if response_details['card_type'] == 'list':
        validate_list(channel,response_details)
        list_json = render_list(channel,response_details)
        current_context['response_queue'].append(list_json)
    elif response_details['card_type'] == 'quick_reply':
        validate_quick_reply(channel,response_details)
        quick_reply_json = render_quick_reply(channel,response_details)
        current_context['response_queue'].append(quick_reply_json)
    elif response_details['card_type'] == 'text':
        text = response_details['message']
        current_context['response_queue'].append(str(text))
    elif response_details['card_type'] == 'file':
        file_json = render_file(channel,response_details)
        current_context['response_queue'].append(file_json) 
    elif response_details['card_type'] == 'image':
        img_json = render_image(channel,response_details)
        current_context['response_queue'].append(img_json)
    elif response_details['card_type'] == 'sticker':
        sticker_json = render_sticker(channel,response_details)
        current_context['response_queue'].append(sticker_json)
    elif response_details['card_type'] == 'catalogue':
        catalogue_json = render_catalogue(channel,response_details)
        current_context['response_queue'].append(catalogue_json)
    elif response_details['card_type'] == 'generic_card':
        generic_card_json = render_generic_card(channel,response_details)
        current_context['response_queue'].append(generic_card_json)
    return

def validate_list(channel,response_details):
    if channel == 'whatsapp':
        len_title = len(response_details['main_title'])
        if len_title > 60:
            raise Exception("main_title has exceeded the character limit of 60!")
        len_body = len(response_details['main_body'])
        if len_body > 1024:
            raise Exception("main_body has exceeded the character limit of 1024!")
        len_button = len(response_details['button_text'])
        if len_button > 20:
            raise Exception("button_text has exceeded the character limit of 20!")
        for section in response_details['sections']:
            if 'title' not in section.keys():
                raise Exception("Section does not have a Title!")
            for option in section['options']:
                len_opt_title = len(option['title'])
                if len_opt_title > 24:
                    raise Exception(f"'{option['title']}' has exceeded the character limit of 24!")
                if 'description' in option.keys():
                    len_desc = len(option['description'])
                    if len_desc > 72:
                        raise Exception(f"'{option['description']}' has exceeded the character limit of 72!")
    return

def render_list(channel,response_details):
    list_json = {}
    if channel == 'whatsapp':
        main_title = response_details['main_title']
        main_body = response_details['main_body']
        button_text = response_details['button_text']
        section_list = response_details['sections']
        item_list = []
        for section in section_list:
            section_body = {}
            section_body['title'] = section['title']
            elements = section['options']
            options_list = []
            for element in elements:
                options = {}
                options['type'] = 'text'
                options['title'] = element['title']
                if 'description' in element.keys():
                    options['description'] = element['description']
                if 'postbackText' in element.keys():
                    options['postbackText'] = element['postbackText']
                options_list.append(options)
            section_body['options'] = options_list
            item_list.append(section_body)
        list_json['type'] = 'list'
        list_json['title'] = main_title
        list_json['body'] = main_body
        list_json['msgid'] = 'lst1'
        list_json['globalButtons'] = []
        list_json['globalButtons'].append({"type":"text","title":str(button_text)})
        list_json['items'] = item_list
    elif channel == 'gip':
        list_json['type'] = 'list'
        if "topElementStyle" in response_details.keys():
            list_json['topElementStyle'] = response_details['topElementStyle']
        else:
            list_json['topElementStyle'] = 'compact'
        if "msgid" in response_details.keys():
            list_json['msgid'] = response_details['msgid']
        else:
            list_json['msgid'] = "list123"
        list_json['items'] = response_details['items']
        list_json['globalButtons'] = response_details['globalButtons']
    return list_json

def validate_quick_reply(channel,response_details):
    if channel == 'whatsapp':
        if 'header' in response_details.keys():
            len_header = len(response_details['header'])
            if len_header > 60:
                raise Exception("header has exceeded the character limit of 60!")
        if 'caption' in response_details.keys():
            len_caption = len(response_details['caption'])
            if len_caption > 60:
                raise Exception("cation has exceeded the character limit of 60!")
        len_options = len(response_details['options'])
        if len_options > 3:
            raise Exception("You cannot use more than 3 quick reply buttons in whatsapp!")
    return

def render_quick_reply(channel,response_details):
    quick_reply_json = {}
    if channel == 'whatsapp':
        buttons = response_details['options']
        options_list = []
        for button in buttons:
            button_dict = {}
            button_dict['type'] = 'text'
            button_dict['title'] = str(button)
            options_list.append(button_dict)
        quick_reply_json['type'] = 'quick_reply'
        quick_reply_json['content'] = {}
        quick_reply_json['content']['type'] = 'text'
        if 'content_type' in response_details.keys():
            quick_reply_json['content']['type'] = response_details['content_type']
        if 'header' in response_details.keys():
            quick_reply_json['content']['header'] = response_details['header']
        quick_reply_json['content']['text'] = response_details['body']
        if 'caption' in response_details.keys():
            quick_reply_json['content']['caption'] = response_details['caption']
        if 'url' in response_details.keys():
            quick_reply_json['content']['url'] = response_details['url']
        quick_reply_json['msgid'] = 'qr1'
        quick_reply_json['options'] = options_list
    elif channel == 'gip':
        quick_reply_json['type'] = 'quick_reply'
        quick_reply_json['content'] = {}
        quick_reply_json['content']['type'] = 'text'
        if 'content_type' in response_details.keys():
            quick_reply_json['content']['type'] = response_details['content_type']
        if 'url' in response_details.keys():
            quick_reply_json['content']['url'] = response_details['url']
        if quick_reply_json['content']['type'] == 'text':
            quick_reply_json['content']['text'] = response_details['text']
        quick_reply_json['msgid'] = 'qr1'
        options_list = []
        for button in response_details['options']:
            button_dict = {}
            button_dict['type'] = 'text'
            button_dict['title'] = button['title']
            if 'url' in button.keys():
                button_dict['iconurl'] = button['url']
            options_list.append(button_dict)
        quick_reply_json['options'] = options_list
    elif channel == 'instagram':
        quick_reply_json['type'] = 'quick_reply'
        quick_reply_json["msgid"] = "qr1"
        content = {}
        content['type'] = 'text'
        content['text'] = response_details['text']
        quick_reply_json['content'] = content
        quick_reply_json['options'] = response_details['options']
    return quick_reply_json

def render_file(channel,response_details):
    file_json = {}
    if channel == 'whatsapp':
        file_json['type'] = 'file'
        file_json['url'] = response_details['url']
        file_json['filename'] = response_details['filename']
    elif channel == 'gip':
        file_json['type'] = 'file'
        file_json['url'] = response_details['url']
        file_json['text'] = response_details['text']       
    return file_json

def render_image(channel,response_details):
    image_json = {}
    if channel == 'whatsapp':
        image_json['type'] = 'image'
        image_json['originalUrl'] = response_details['url']
        image_json['previewUrl'] = response_details['url']
        image_json['caption'] = response_details['caption']
    elif channel == 'gip':
        image_json['type'] = 'image'
        image_json['originalUrl'] = response_details['url']
        image_json['previewUrl'] = response_details['url']
    elif channel == 'instagram':
        image_json['type'] = 'image'
        image_json['originalUrl'] = response_details['url']
        image_json['previewUrl'] = response_details['url']             
    return image_json

def render_sticker(channel,response_details):
    sticker_json = {}
    if channel == 'whatsapp':
        sticker_json['type'] = 'sticker'
        sticker_json['url'] = response_details['url']
    elif channel == 'gip':
        sticker_json['type'] = 'sticker'
        sticker_json['url'] = response_details['url']
    elif channel == 'instagram':
        sticker_json['type'] = 'sticker'
        sticker_json['id'] = response_details['id']
    return sticker_json         

def render_generic_card(channel,response_details):
    generic_json = {}
    if channel == 'whatsapp':
        raise Exception('Generic Cards are not supported in Whatsapp!')
    elif channel == 'gip':
        generic_json['type'] = 'generic_card'
        generic_json['mml'] = {}
        generic_json['mml'] = response_details
        generic_json['mml']['type'] = 'generic_card'        
    return generic_json

def render_catalogue(channel,response_details):
    catalogue_json = {}
    if channel == 'whatsapp':
        raise Exception('Catalogues are not supported in Whatsapp yet!')
    elif channel == 'gip':
        catalogue_json = response_details
        catalogue_json['type'] = 'catalogue'
    elif channel == 'instagram':
        catalogue_json = response_details
        catalogue_json['type'] = 'catalogue'      
    return catalogue_json    

def behave_extract_bot_response_text(channel,bot_response):
    if channel == 'whatsapp':
        bot_text = ''
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] != '{':
                    bot_text = resp['text']
                    break
        else:
            bot_text = bot_response[0]['text']
        return bot_text
    if channel == 'gip':
        bot_text = ''
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] != '{':
                    bot_text = resp['text']
                    break
        else:
            bot_text = bot_response[0]['text']
        return bot_text
    if channel == 'instagram':
        bot_text = ''
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] != '{':
                    bot_text = resp['text']
                    break
        else:
            bot_text = bot_response[0]['text']
        return bot_text

def behave_compare_text(channel,input_text,bot_text):
    if input_text == bot_text:
        return True
    return False

def behave_extract_bot_response_quick_reply(channel,bot_response):
    if channel == 'whatsapp':
        bot_text = ''
        bot_buttons = []
        bot_header_url = ''
        bot_caption = ''
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] == '{':
                    qr_json = loads(resp['text'])
                    bot_text = qr_json['content']['text']
                    if qr_json['content']['type'] != 'text':
                        bot_header_url = qr_json['content']['url']
                    else:
                        if 'header' in qr_json['content'].keys():
                            bot_header_url = qr_json['content']['header']
                    if 'caption' in qr_json['content'].keys():
                        bot_caption = qr_json['content']['caption']
                    for option in qr_json['options']:
                        bot_buttons.append(option['title'])  
                    break
        else:
            qr_json = loads(bot_response[0]['text'])
            bot_text = qr_json['content']['text']
            if qr_json['content']['type'] != 'text':
                bot_header_url = qr_json['content']['url']
            else:
                if 'header' in qr_json['content'].keys():
                    bot_header_url = qr_json['content']['header']
            if 'caption' in qr_json['content'].keys():
                bot_caption = qr_json['content']['caption']
            for option in qr_json['options']:
                bot_buttons.append(option['title'])
        return (bot_text.strip(),bot_buttons.sort(),bot_header_url.strip(),bot_caption.strip())
    if channel == 'gip':
        bot_content = ''
        bot_buttons = []
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] == '{':
                    qr_json = loads(resp['text'])
                    if qr_json['content']['type'] == 'text':
                        bot_content = qr_json['content']['text']
                    else:
                        bot_content = qr_json['content']['url']
                    for option in qr_json['options']:
                        bot_buttons.append(option)  
                    break
        else:
            qr_json = loads(bot_response[0]['text'])
            if qr_json['content']['type'] == 'text':
                bot_content = qr_json['content']['text']
            else:
                bot_content = qr_json['content']['url']
            for option in qr_json['options']:
                del option['type']
                bot_buttons.append(option) 
        return (bot_content.strip(),bot_buttons)
    if channel == 'instagram':
        bot_text = ''
        bot_buttons = []
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] == '{':
                    qr_json = loads(resp['text'])
                    bot_text = qr_json['content']['text']
                    for option in qr_json['options']:
                        bot_buttons.append(option['title'])  
                    break
        else:
            qr_json = loads(bot_response[0]['text'])
            bot_text = qr_json['content']['text']
            for option in qr_json['options']:
                bot_buttons.append(option)
        return (bot_text.strip(),bot_buttons.sort()) 

def behave_compare_quick_reply(channel,input_content,bot_resp):
    if channel == 'whatsapp':
        input_message = input_content['message'].strip()
        button_input = input_content['buttons'].sort()
        input_content_type = 'text'
        input_header_url = ''
        input_caption = ''
        if 'content_type' in input_content.keys():
            input_content_type = input_content['content_type']
        if input_content_type == 'text':
            if 'header' in input_content.keys():
                input_header_url = input_content['header']
        else:
            input_header_url = input_content['url']
        if 'caption' in input_content.keys():
            input_caption = input_content['caption']
        bot_message = bot_resp[0]
        bot_buttons = bot_resp[1]
        bot_header_url = bot_resp[2]
        bot_caption = bot_resp[3]
        if (input_message != bot_message):
            return False, "Message comparison failed!"
        if (button_input != bot_buttons):
            return False, "Buttons did not match!"
        if (input_header_url != '') and (input_header_url.strip() != bot_header_url):
            return False, "Header/URL did not match!"
        if (input_caption != '') and (input_caption.strip() != bot_caption):
            return False, "Caption did not match!"        
        return True, None
    elif channel == 'gip':
        input_content_type = input_content['content_type']
        input_buttons = input_content['buttons']
        bot_content = bot_resp[0]
        bot_buttons = bot_resp[1]
        qr_content = ''
        if input_content_type == 'text':
            qr_content = input_content['text']
        else:
            qr_content = input_content['url']
        for button in input_buttons:
            if 'url' in button.keys():
                if button['url'].lower() in ['n.a.']:
                    del button['url']
                else:
                    button['iconurl'] = button.pop('url')
        if (qr_content != bot_content):
            return False, "Mismatch in content dict!"
        for button in input_buttons:
            if button not in bot_buttons:
                return False, "Buttons did not match!"
        return True, None
    elif channel == 'instagram':
        input_message = input_content['message'].strip()
        button_input = input_content['buttons'].sort()
        bot_message = bot_resp[0]
        bot_buttons = bot_resp[1]
        if (input_message != bot_message):
            return False, "Message comparison failed!"
        if (button_input != bot_buttons):
            return False, "Buttons did not match!"
        return True, None

def behave_extract_bot_response_list(channel,bot_response):
    if channel == 'whatsapp':
        bot_text = ''
        bot_list = []
        bot_header = ''
        bot_global_button = ''
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] == '{':
                    list_json = loads(resp['text'])
                    bot_text = list_json['body']
                    bot_header = list_json['title']
                    bot_global_button = list_json['globalButtons']
                    for item in list_json['items']:
                        list_element = {}
                        list_element["Category"] = item['title']
                        list_element["Items"] = []
                        for option in item['options']:
                            list_element["Items"].append(option['title'])
                        bot_list.append(list_element)
                    break
        else:
            list_json = loads(bot_response[0]['text'])
            bot_text = list_json['body']
            bot_header = list_json['title']
            bot_global_button = list_json['globalButtons']
            for item in list_json['items']:
                list_element = {}
                list_element["Category"] = item['title']
                list_element["Items"] = []
                for option in item['options']:
                    list_element["Items"].append(option['title'])
                bot_list.append(list_element)
        return (bot_text.strip(),bot_list,bot_header.strip(),bot_global_button)
    elif channel == 'gip':
        bot_items = []
        bot_buttons = []
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] == '{':
                    list_json = loads(resp['text'])
                    for item in list_json['items']:
                        bot_items.append(item)
                    for option in list_json['globalButtons']:
                        bot_buttons.append(option)
                    break
        else:
            list_json = loads(bot_response[0]['text'])
            for item in list_json['items']:
                bot_items.append(item)
            for option in list_json['globalButtons']:
                bot_buttons.append(option)           
        return (bot_items,bot_buttons)  

def behave_compare_list(channel,input_content,bot_resp):
    if channel == 'whatsapp':
        input_message = input_content['message'].strip()
        list_input = input_content['list']
        input_header = ''
        input_global_button = ''
        if 'header' in input_content.keys():
            input_header = input_content['header'].strip()
        if 'globalButtons' in input_content.keys():
            input_global_button = input_content['globalButtons']
        bot_message = bot_resp[0]
        bot_list = bot_resp[1]
        bot_header = bot_resp[2]
        bot_buttons = bot_resp[3]
        list_comparison = True
        for category in list_input:
            category['Items'].sort()
        for category in bot_list:
            category['Items'].sort()
        for element in list_input:
            if element not in bot_list:
                list_comparison = False
        for element in bot_list:
            if element not in list_input:
                list_comparison = False
        if (input_message != bot_message):
            return False, "Message comparison failed!"
        if (list_comparison == False):
            return False, "List items did not match!"
        if (input_header != '') and (input_header != bot_header):
            return False, "Headers did not match!"
        if (input_global_button != '') and (input_global_button != bot_buttons):
            return False, "Global Buttons did not match!"
        return True, None
    if channel == 'gip':
        input_items = input_content['items']
        input_buttons = input_content['globalButtons']
        bot_items = bot_resp[0]
        bot_buttons = bot_resp[1]
        list_comparison = True
        default_actions_removed = []
        # remove N.A. default actions and webview_height_ratio in options
        for item in input_items:
            item['title'] = item['title'].encode().decode('unicode_escape')
            item['subtitle'] = item['subtitle'].encode().decode('unicode_escape')
            item['options'] = item.pop('buttons')
            item['options'] = sorted(item['options'], key=lambda d: d['title'])
            if 'url' in item.keys():
                item['imgurl'] = item.pop('url')
            if 'defaultAction' in item.keys():
                if item['defaultAction'] == 'N.A.':
                    del item['defaultAction']
                    default_actions_removed.append(item['title'])
            else:
                default_actions_removed.append(item['title'])
        for item in bot_items:
            item['title'] = item['title'].encode().decode('unicode_escape')
            item['subtitle'] = item['subtitle'].encode().decode('unicode_escape')
            item['options'] = sorted(item['options'], key=lambda d: d['title'])
            if item['title'] in default_actions_removed:
                if 'defaultAction' in item.keys():
                    del item['defaultAction']
            for option in item['options']:
                if 'webview_height_ratio' in option.keys():
                    del option['webview_height_ratio']
        # compare items
        for item in input_items:
            if item not in bot_items:
                return False, "Item Comparison Failed!"
        # compare buttons/options
        input_buttons = sorted(input_buttons, key=lambda d: d['title'])
        bot_buttons = sorted(bot_buttons, key=lambda d: d['title'])
        if (input_buttons != bot_buttons):
            return False, "Button Comaparison Failed!"
        return True, None

def behave_extract_bot_response_file(channel,bot_response):
    if channel == 'whatsapp':
        file_json = loads(bot_response[0]['text'])
        bot_filename = file_json['filename']
        bot_url = file_json['url']
        return (bot_filename,bot_url)
    elif channel == 'gip':
        file_json = loads(bot_response[0]['text'])
        bot_text = file_json['text']
        bot_url = file_json['url']
        return (bot_text,bot_url)

def behave_compare_file(channel,input_content,bot_resp):
    if channel == 'whatsapp':
        input_filename = input_content['filename']
        input_url = input_content['url']
        bot_filename = bot_resp[0]
        bot_url = bot_resp[1]
        if (input_filename != bot_filename):
            return False, "Filename did not match!"
        if (input_url != bot_url):
            return False, "URLs did not match!"
        return True, None
    elif channel == 'gip':
        input_text = input_content['text']
        input_url = input_content['url']
        bot_filename = bot_resp[0]
        bot_url = bot_resp[1]
        if (input_text != bot_filename): 
            return False, "Text did not match!"
        if (input_url != bot_url):
            return False, "URLs did not match!"
        return True, None       

def behave_extract_bot_response_image(channel,bot_response):
    if channel == 'whatsapp':
        image_json = loads(bot_response[0]['text'])
        bot_caption = image_json['caption']
        bot_url = image_json['originalUrl']
        return (bot_caption,bot_url)
    elif channel == 'gip':
        image_json = loads(bot_response[0]['text'])
        bot_url = image_json['originalUrl']
        return bot_url,None
    elif channel == 'instagram':
        image_json = loads(bot_response[0]['text'])
        bot_url = image_json['originalUrl']
        return bot_url,None

def behave_compare_image(channel,input_content,bot_resp):
    if channel == 'whatsapp':
        input_url = input_content['url']
        input_caption = input_content['caption']
        bot_caption = bot_resp[0]
        bot_url = bot_resp[1]
        if (input_caption != bot_caption):
            return False, "Caption did not match!"
        if (input_url != bot_url):
            return False, "URLs did not match!"
        return True, None
    elif channel == 'gip':
        input_url = input_content['url']
        bot_url = bot_resp[0]
        if (input_url != bot_url):
            return False, "URLs did not match!"
        return True, None
    elif channel == 'instagram':
        input_url = input_content['url']
        bot_url = bot_resp[0]
        if (input_url != bot_url):
            return False, "URLs did not match!"
        return True, None

def behave_extract_bot_response_sticker(channel,bot_response):
    if channel == 'whatsapp':
        sticker_json = loads(bot_response[0]['text'])
        bot_url = sticker_json['url']
        return bot_url
    elif channel == 'gip':
        sticker_json = loads(bot_response[0]['text'])
        bot_url = sticker_json['url']
        return bot_url
    elif channel == 'instagram':
        sticker_json = loads(bot_response[0]['text'])
        bot_id = sticker_json['id']
        return bot_id

def behave_compare_sticker(channel,input_content,bot_resp):
    if channel == 'whatsapp':
        input_url = input_content['url']
        bot_url = bot_resp
        if (input_url != bot_url):
            return False, "URLs did not match!"
        return True, None
    elif channel == 'gip':
        input_url = input_content['url']
        bot_url = bot_resp
        if (input_url != bot_url):
            return False, "URLs did not match!"
        return True, None
    elif channel == 'instagram':
        input_url = input_content['id']
        bot_url = bot_resp
        if (input_url != bot_url):
            return False, "IDs did not match!"
        return True, None 

def behave_extract_bot_response_catalogue(channel,bot_response):
    if channel == 'whatsapp':
        raise Exception('Catalogue not supported in Whatsapp yet!')
    elif channel == 'gip':
        bot_items = []
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] == '{':
                    cat_json = loads(resp['text'])
                    for item in cat_json['items']:
                        bot_items.append(item)
                    break
        else:
            cat_json = loads(bot_response[0]['text'])
            for item in cat_json['items']:
                bot_items.append(item)           
        return bot_items
    elif channel == 'instagram':
        bot_items = []
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] == '{':
                    cat_json = loads(resp['text'])
                    for item in cat_json['items']:
                        bot_items.append(item)
                    break
        else:
            cat_json = loads(bot_response[0]['text'])
            for item in cat_json['items']:
                bot_items.append(item)           
        return bot_items    

def behave_compare_catalogue(channel,input_content,bot_resp):
    if channel == 'whatsapp':
        raise Exception('Catalogue not supported in Whatsapp yet!')
    if channel == 'gip':
        input_items = input_content['items']
        bot_items = bot_resp
        default_actions_removed = []
        # remove N.A. default actions and webview_height_ratio in options
        for item in input_items:
            item['title'] = item['title'].encode().decode('unicode_escape')
            item['subtitle'] = item['subtitle'].encode().decode('unicode_escape')
            item['options'] = item.pop('buttons')
            item['options'] = sorted(item['options'], key=lambda d: d['title'])
            if 'url' in item.keys():
                item['imgurl'] = item.pop('url')
            if 'defaultAction' in item.keys():
                if item['defaultAction'] == 'N.A.':
                    del item['defaultAction']
                    default_actions_removed.append(item['title'])
            else:
                default_actions_removed.append(item['title'])
        for item in bot_items:
            item['title'] = item['title'].encode().decode('unicode_escape')
            item['subtitle'] = item['subtitle'].encode().decode('unicode_escape')
            item['options'] = sorted(item['options'], key=lambda d: d['title'])
            if item['title'] in default_actions_removed:
                if 'defaultaction' in item.keys():
                    del item['defaultaction']
            for option in item['options']:
                if 'webview_height_ratio' in option.keys():
                    del option['webview_height_ratio']
        # compare items
        for item in input_items:
            if item not in bot_items:
                title = item['title']
                fstr = f"Item with title {title} did not match!"
                return False, fstr
        return True, None
    if channel == 'instagram':
        input_items = input_content['items']
        bot_items = bot_resp
        default_actions_removed = []
        # remove N.A. default actions and webview_height_ratio in options
        for item in input_items:
            item['title'] = item['title'].encode().decode('unicode_escape')
            item['subtitle'] = item['subtitle'].encode().decode('unicode_escape')
            item['options'] = item.pop('buttons')
            item['options'] = sorted(item['options'], key=lambda d: d['title'])
            if 'url' in item.keys():
                item['imgurl'] = item.pop('url')
            if 'defaultAction' in item.keys():
                if item['defaultAction'] == 'N.A.':
                    del item['defaultAction']
                    default_actions_removed.append(item['title'])
                else:
                    item['defaultaction'] = item.pop('defaultAction')
            else:
                default_actions_removed.append(item['title'])
        for item in bot_items:
            item['title'] = item['title'].encode().decode('unicode_escape')
            item['subtitle'] = item['subtitle'].encode().decode('unicode_escape')
            item['options'] = sorted(item['options'], key=lambda d: d['title'])
            if item['title'] in default_actions_removed:
                if 'defaultaction' in item.keys():
                    del item['defaultaction']
            for option in item['options']:
                if 'webview_height_ratio' in option.keys():
                    del option['webview_height_ratio']
        # compare items
        for item in input_items:
            if item not in bot_items:
                title = item['title']
                fstr = f"Item with title {title} did not match!"
                return False, fstr
        return True, None

def behave_extract_bot_response_generic_card(channel,bot_response):
    if channel == 'whatsapp':
        raise Exception("Generic Cards are not supported on Whatsapp!")
    elif channel == 'gip':
        bot_dict = {}
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] == '{':
                    gencard = loads(resp['text'])
                    bot_dict = gencard['mml']
                    break
        else:
            gencard = loads(bot_response[0]['text'])
            bot_dict = gencard['mml']           
        return bot_dict

def behave_compare_generic_card(channel,input_content,bot_resp):
    if channel == 'whatsapp':
        raise Exception("Generic Cards are not supported on Whatsapp!")
    elif channel == 'gip':
        gencard_input = {}
        gencard_input['message'] = input_content['message']
        i=0
        for title in input_content['titles']:
            key_str = 'title_' + str(i)
            gencard_input[key_str] = title
            i = i+1
        if 'core_entity' in input_content.keys():
            if (type(input_content['core_entity']) == str):
                if input_content['core_entity'] != 'N.A.':
                    gencard_input['core_entity'] = input_content['core_entity']
            else:
                gencard_input['core_entity'] = input_content['core_entity']['text']
                gencard_input['action0_rank'] = 0
                gencard_input['action0_type'] = int(input_content['core_entity']['type'])
                gencard_input['action0_to'] = input_content['core_entity']['url']
                if gencard_input['action0_type'] == 4:
                    gencard_input['action0_content'] = gencard_input['core_entity']
                    gencard_input['action0_to'] = gencard_input['core_entity']
                else:
                    gencard_input['action0_content'] = gencard_input['action0_to']
        j=1
        for action in input_content['actions']:
            key_rank = 'action' + str(j) + '_rank'
            key_type = 'action' + str(j) + '_type'
            key_to = 'action' + str(j) + '_to'
            key_content = 'action' + str(j) + '_content'
            gencard_input[key_rank] = 1
            gencard_input[key_type] = action['type']
            gencard_input[key_to] = action['to']
            gencard_input[key_content] = action['content']
            j=j+1
        for key,value in gencard_input.items():
            if key not in bot_resp.keys():
                reason = f"Key={key} not found in bot response!"
                return False, reason
            if gencard_input[key] != bot_resp[key]:
                reason = f"Values for Key={key} did not match!"
                return False, reason
        return True, None

def behave_extract_bot_response_generic_card_media(channel,bot_response):
    if channel == 'whatsapp':
        raise Exception("Generic Cards are not supported on Whatsapp!")
    elif channel == 'gip':
        bot_dict = {}
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] == '{':
                    gencard = loads(resp['text'])
                    bot_dict = gencard['mml']
                    break
        else:
            gencard = loads(bot_response[0]['text'])
            bot_dict = gencard['mml']           
        return bot_dict

def behave_compare_generic_card_media(channel,input_content,bot_resp):
    if channel == 'whatsapp':
        raise Exception("Generic Cards are not supported on Whatsapp!")
    elif channel == 'gip':
        gencard_input = {}
        gencard_input['message'] = input_content['message']
        i=0
        for title in input_content['titles']:
            key_str = 'title_' + str(i)
            gencard_input[key_str] = title
            i = i+1
        if 'media' in input_content.keys():
            if (type(input_content['media']) == str):
                if input_content['media'] != 'N.A.':
                    gencard_input['media'] = input_content['media']
            else:
                gencard_input['media1_type'] = int(input_content['media']['type'])
                gencard_input['media1_url'] = input_content['media']['url']
                if gencard_input['media1_type'] != 1:
                    gencard_input['media1_cover'] = gencard_input['media']['cover']
        j=1
        for action in input_content['actions']:
            key_rank = 'action' + str(j) + '_rank'
            key_type = 'action' + str(j) + '_type'
            key_to = 'action' + str(j) + '_to'
            key_content = 'action' + str(j) + '_content'
            gencard_input[key_rank] = 1
            gencard_input[key_type] = action['type']
            gencard_input[key_to] = action['to']
            gencard_input[key_content] = action['content']
            j=j+1
        for key,value in gencard_input.items():
            if key not in bot_resp.keys():
                reason = f"Key={key} not found in bot response!"
                return False, reason
            if gencard_input[key] != bot_resp[key]:
                reason = f"Values for Key={key} did not match!"
                return False, reason
        return True, None

    



from .response_cards import show_response_card

import difflib
import warnings
warnings.filterwarnings('ignore')
from fuzzywuzzy import fuzz
from  . import gs_global

def format_response_card_quick_reply(gs_context, current_context, response):
#     print(response)
    response_json_quick = {}
    response_json_quick['content_type'] = 'image'
    response_json_quick["card_type"] = "quick_reply"
    response_json_quick["header"] = ""
    response_json_quick["body"] = response["title"]
    response_json_quick["caption"] = ""
    if response['url']!='None':
        response_json_quick['url'] = response["url"]
    else:
        response_json_quick['content_type'] = 'text'
        response_json_quick['url'] = ''
    filter_buttons = ['Invalid-choice-message', 'prompt','Value_Description',
                      'card_type_gip','card_type_instagram','card_type_whatsapp', 'List_Header', 'List_Category', 'url']
    response["options"] = [x for x in response['options'] if x not in filter_buttons]
    response_json_quick["options"] = response["options"]
#     print('45121',response["options"])
#     print(response_json_quick)
    show_response_card(gs_context, current_context, response_json_quick)


def format_response_card_list(gs_context, current_context, response, temp_menu):
#     print(response)
    channel = gs_context['channel']
    response_json_list = {}
    # print(temp_menu)
    response_json_list['card_type'] = temp_menu['card_type_{}'.format(channel)]
    response_json_list['main_title'] = temp_menu['List_Header']
    response_json_list['main_body'] = response['title']
    response_json_list['button_text'] = "Show list"
    response_json_list["sections"] = []
    response_json_list_cat1 = {}
    response_json_list_cat1["title"] = temp_menu['List_Category']
    options = []
    filter_buttons = ['Invalid-choice-message', 'prompt','Value_Description',
                      'card_type_gip','card_type_instagram','card_type_whatsapp', 'List_Header', 'List_Category', 'url']
    response["options"] = [x for x in response['options'] if x not in filter_buttons]
    if response['option_desc']!='None':    
        lookup_desc_dict = response['option_desc']
    else:
        lookup_desc_dict = {}
    for button in response["options"]:
        each_option = {}
        each_option["title"] = button
        try:
            each_option['description'] = lookup_desc_dict[button]
        except:
            pass
        options.append(each_option)
    response_json_list_cat1["options"] = options
#     print('451212',response["options"])
    response_json_list["sections"].append(response_json_list_cat1)
    show_response_card(gs_context, current_context, response_json_list)
#     print(response_json_list)
# 
'''Additional thing for gip'''

def create_items_list_url(title,url):
    sample = {
                "title": title,
                "subtitle": "",
                "options": [
                    {
                        "title": title,
                        "type": "url",
                        "url": url
                    }
                ]
            }
    return sample

def create_items_list_text(title,description):
    sample = {
                "title": title,
                "subtitle": "",
                "options": [
                    {
                        "title": description,
                        "type": "text"
                    }
                ]
            }
    return(sample)

def response_card_distributor(gs_context, current_context, response, temp_menu):
    channel = gs_context['channel']
    if gs_context['channel']=='gip':
        if temp_menu['card_type_{}'.format(channel)] == 'list':
            format_gip_list_card(gs_context, current_context, response, temp_menu)
        elif temp_menu['card_type_{}'.format(channel)] == 'quick_reply':
            format_gip_quick_reply_card(gs_context, current_context, response, temp_menu)
        # elif temp_menu['card_type_{}'.format(channel)] == 'file':
        #     format_gip_file_card(gs_context, current_context, response, temp_menu)
        # elif temp_menu['card_type_{}'.format(channel)] == 'sticker':
        #     format_gip_sticker_card(gs_context, current_context, response, temp_menu)
        elif temp_menu['card_type_{}'.format(channel)] == 'catalogue':
            format_gip_catalogue_card(gs_context, current_context, response, temp_menu)
        elif temp_menu['card_type_{}'.format(channel)] == 'generic_card':
            format_gip_generic_card(gs_context, current_context, response, temp_menu)
    elif gs_context['channel']=='whatsapp':
        if temp_menu['card_type_{}'.format(channel)] == 'list':
            format_wapp_list_card(gs_context, current_context, response, temp_menu)
        elif temp_menu['card_type_{}'.format(channel)] == 'quick_reply':
            format_wapp_quick_reply_card(gs_context, current_context, response, temp_menu)
        # elif temp_menu['card_type_{}'.format(channel)] == 'file':
        #     format_wapp_file_card(gs_context, current_context, response, temp_menu) 
        # elif temp_menu['card_type_{}'.format(channel)] == 'sticker':
        #     format_wapp_sticker_card(gs_context, current_context, response, temp_menu)       
    elif gs_context['channel']=='instagram':
#         if temp_menu['card_type_{}'.format(channel)] == 'list':
#             format_insta_list_card(gs_context, current_context, response, temp_menu)
        if temp_menu['card_type_{}'.format(channel)] == 'quick_reply':
            format_insta_image_quick_reply(gs_context, current_context, response, temp_menu)
#         elif temp_menu['card_type_{}'.format(channel)] == 'file':
#             format_insta_file_card(gs_context, current_context, response, temp_menu)
        # elif temp_menu['card_type_{}'.format(channel)] == 'sticker':
        #     format_insta_sticker_card(gs_context, current_context, response, temp_menu)
        elif temp_menu['card_type_{}'.format(channel)] == 'catalogue':
            format_insta_catalogue_card(gs_context, current_context, response, temp_menu)
#         elif temp_menu['card_type_{}'.format(channel)] == 'generic_card':
#             format_insta_generic_card(gs_context, current_context, response, temp_menu)
    # if temp_menu['card_type_{}'.format(channel)] == 'image':
    #     format_all_image_card(gs_context, current_context, response, temp_menu)
        

def format_gip_list_card(gs_context, current_context, response, temp_menu):
    response_json_gip_list = {}
    response_json_gip_list['globalbuttons'] = {}
    response_json_gip_list['card_type'] = "list"
    response_json_gip_list['globalbuttons']['type'] = 'text'
    response_json_gip_list['globalbuttons']['title'] = "View More"
    response_json_list_cat1 = {}
    options = []
    response_json_list_cat1["title"] = temp_menu['list_header']
    filter_buttons = ['Invalid-choice-message', 'prompt','Value_Description',
                      'card_type_gip','card_type_instagram','card_type_whatsapp', 'List_Header', 'List_Category', 'url']
    response["options"] = [x for x in response['options'] if x not in filter_buttons]        
    if response['option_desc']!='None':    
        lookup_desc_dict = response['option_desc']
    else:
        lookup_desc_dict = {}
    response_json_gip_list['items'] = []
    for button in response["options"]:
        each_option = {}
        each_option["title"] = button
        try:
            each_option['description'] = lookup_desc_dict[button]
        except:
            pass
        options.append(each_option)
    
    for option in options:
        print(option)
        response_json_gip_list['items'].append(create_items_list_text(option['title'], option['description']))
    show_response_card(gs_context, current_context, response_json_gip_list)
    print(response_json_gip_list)

def format_gip_quick_reply_card(gs_context, current_context, response, temp_menu):
    response_json_quick = {}
    response_json_quick['content_type'] = 'text'
    response_json_quick["card_type"] = "quick_reply"
    response_json_quick["text"] = response["title"]
    if response['url'] != 'None':
        response_json_quick['url'] = response["url"]
    else:
        response_json_quick['content_type'] = 'text'
        response_json_quick['url'] = ''
    filter_buttons = ['Invalid-choice-message', 'prompt','Value_Description',
                      'card_type_gip','card_type_instagram','card_type_whatsapp', 'List_Header', 'List_Category', 'url', 'bot_header']
    response["options"] = [
        x for x in response['options'] if x not in filter_buttons]
    lst = []
    for i in response["options"]:
        dct = {}
        dct["title"] = i
        lst.append(dct)
    response_json_quick["options"] = lst
    print(f"response_json_quick......................{response_json_quick}")
    show_response_card(gs_context, current_context, response_json_quick)
    print(response_json_quick)

def format_gip_catalogue_card(gs_context, current_context, response, temp_menu,deals_df):
    response_json_list = {}
    response_json_list['card_type'] = temp_menu['card_type_gip']
    response_json_list['items'] = []
    deal_end_count = min(7,deals_df.shape[0])
    response_json_list["options"] = []
    for deals in range(deal_end_count):
        temp_dict = {}
        temp_dict['title'] = deals_df.iloc[deals,:]['title']
        temp_dict['subtitle'] = deals_df.iloc[deals,:]['long_offer']
        temp_dict['imgurl'] = deals_df.iloc[deals,:]['image_url']
        temp_dict['options'] = [{'title':'View Details','type':'url','url':deals_df.iloc[deals,:]['url']},
                               {'title':'Buy','type':'text'}]
        response_json_list['items'].append(temp_dict)
    gs_context['past_conversation'] = {'count_deals':deal_end_count,'prev_intent':'deals','deals_df':deals_df.to_dict('dict')}
    show_response_card(gs_context, current_context, response_json_list)
    print(response_json_list)


def format_gip_generic_card(gs_context, current_context, response, temp_menu):
    response_json_gip = {}
    response_json_gip['card_type'] = "generic_card"
    print(f"response..........................{response}")
    if response['url']!='None':
        response_json_gip['url'] = response["url"]
    else:
        response_json_gip['url'] = ''
    response_json_gip['title_0'] = temp_menu["bot_header"]
    response_json_gip['message'] = response["title"]
    filter_buttons = ['Invalid-choice-message', 'prompt','Value_Description',
                      'card_type_gip','card_type_instagram','card_type_whatsapp', 'List_Header', 'List_Category', 'url', 'bot_header']
    response["options"] = [x for x in response['options'] if x not in filter_buttons]
    for index,option in enumerate(response["options"]):
            response_json_gip['action{}_rank'.format(index)] = 1
            response_json_gip['action{}_type'.format(index)] = 7
            response_json_gip['action{}_content'.format(index)] = option
            response_json_gip['action{}_to'.format(index)] = option
    print(response_json_gip)
    show_response_card(gs_context, current_context, response_json_gip)

##  Wapp

def format_wapp_quick_reply_card(gs_context, current_context, response,temp_menu):
    response_json_quick = {}
    response_json_quick['content_type'] = 'image'
    response_json_quick["card_type"] = "quick_reply"
    response_json_quick["header"] = ""
    response_json_quick["body"] = response["title"]
    response_json_quick["caption"] = ""
    if response['url'] != 'None':
        response_json_quick['url'] = response["url"]
    else:
        response_json_quick['content_type'] = 'text'
        response_json_quick['url'] = ''
    filter_buttons = ['Invalid-choice-message', 'prompt','Value_Description',
                      'card_type_gip','card_type_instagram','card_type_whatsapp', 'List_Header', 'List_Category', 'url']
    response["options"] = [
        x for x in response['options'] if x not in filter_buttons]
    response_json_quick["options"] = response["options"]
    show_response_card(gs_context, current_context, response_json_quick)
    print(response_json_quick)




def format_wapp_list_card(gs_context, current_context, response, temp_menu):
    response_json_list = {}
    response_json_list['card_type'] = temp_menu['card_type_whatsapp']
    response_json_list['main_title'] = temp_menu['prompt']
    response_json_list['main_body'] = response['title']
    response_json_list['button_text'] = "Show list"
    response_json_list["sections"] = []
    response_json_list_cat1 = {}
    response_json_list_cat1["title"] = temp_menu['List_Category']
    options = []
    filter_buttons = ['Invalid-choice-message', 'prompt',
                      'card_type_gip','card_type_instagram','card_type_whatsapp', 'List_Header', 'List_Category', 'url','Value_Description']
    response["options"] = [
        x for x in response['options'] if x not in filter_buttons]
    if response['option_desc'] != 'None':
        lookup_desc_dict = response['option_desc']
    else:
        lookup_desc_dict = {}
    for button in response["options"]:
        each_option = {}
        each_option["title"] = button
        try:
            each_option['description'] = lookup_desc_dict[button]
        except:
            each_option['description'] = " "
        options.append(each_option)
    response_json_list_cat1["options"] = options
    print(9336396175,response_json_list)
    response_json_list["sections"].append(response_json_list_cat1)
    show_response_card(gs_context, current_context, response_json_list)
    
    

## Insta

def format_insta_image_quick_reply(gs_context, current_context, response, temp_menu):
    response_image_insta = {}
    if response['url'] != 'None':
        response_image_insta['url'] = response["url"]
    response_image_insta['card_type'] = 'image'
    response_image_insta['caption'] = 'Caption'
    print(response_image_insta)
    # show_response_card(gs_context, current_context, response_image_insta)
    response_json_quick = {}
    response_json_quick["card_type"] = "quick_reply"
    response_json_quick["text"] = response["title"]
    filter_buttons = ['Invalid-choice-message', 'prompt','Value_Description',
                      'card_type_gip','card_type_instagram','card_type_whatsapp', 'List_Header', 'List_Category', 'url']
    # print(response["options"])
    all_keys = [
        x for x in response['options'] if x not in filter_buttons]
    print('93369',all_keys)
    response_json_quick["options"] = all_keys
    print(response_json_quick)
    show_response_card(gs_context, current_context, response_json_quick)

def format_insta_catalogue_card(gs_context, current_context, response, temp_menu,deals_df):
    response_json_list = {}
    response_json_list['card_type'] = temp_menu['card_type_instagram']
    response_json_list['items'] = []    
    deal_end_count = min(7,deals_df.shape[0])
    response_json_list["options"] = []
    for deals in range(deal_end_count):
        temp_dict = {}
        temp_dict['title'] = deals_df.iloc[deals,:]['title']
        temp_dict['subtitle'] = deals_df.iloc[deals,:]['long_offer']
        temp_dict['imgurl'] = deals_df.iloc[deals,:]['image_url']
        temp_dict['options'] = [{'title':'View Details','type':'url','url':deals_df.iloc[deals,:]['url']},
                               {'title':'Buy','type':'text'}]
        response_json_list['items'].append(temp_dict)
    gs_context['past_conversation'] = {'count_deals':deal_end_count,'prev_intent':'deals','deals_df':deals_df.to_dict('dict')}
    print(response_json_list)
    show_response_card(gs_context, current_context, response_json_list)
    


'''Additional thing for gip'''

def activate_menu(gs_context, current_context, selection):
    '''
        This function activates the handle active menu until
        the menu_handled is true.
    '''
    current_context['current_intent']['user_text'] = selection
    gs_context['active_menu'] = selection
    # gs_context['active_menu_state'].append(selection)
    gs_context['menu_handled'] = True
    handle_active_menu(gs_context, current_context)
    return

# options = ['Entertainment','Fashion','Electronics & Gadgets','Home & Living','Stationary','Kids & Toddlers','Health & Beauty','Online Recharge','Gift Items','Travel']
def partial_match(x,y):
    return(fuzz.ratio(x,y))
def closest_matcher(word,options,gs_context,current_context):
    try:
        closest_match = (difflib.get_close_matches(word, options))[0]
        print(closest_match,word)
        match_score = (fuzz.ratio(closest_match,word))
        print(match_score)
        if match_score>80:
            return(closest_match)
        elif 40<match_score<80:
            response_json ={}
            response_json["card_type"] = "text"
            response_json["message"] = 'Did you mean this {}'.format(closest_match)
            show_response_card(gs_context,current_context,response_json)
            # print(response_json)
            return(closest_match)
        else:
            return(word)
    except:
        return(word)

def handle_active_menu(gs_context,current_context):
    """
        This function will come handy when it is a live dictionary
    """
#     selection = current_context['current_intent']['user_text']
#     print('Print whether the live dictionary is list',isinstance(gs_context['live_dict'][selection],dict))
    if gs_context['menu_handled']:
        channel = gs_context['channel']
        selection = current_context['current_intent']['user_text']
        db = gs_global.global_context['mongo_connection']
        temp_selection = selection.split(" ")
        if len(temp_selection) >1 and temp_selection[-1].isdigit():
            selection = " ".join(temp_selection[:-1])
        if gs_context["slots"]['language']!='en':
            selection = db.reverse_translate(selection)
        response ={}
        if isinstance(gs_context['live_dict'],dict):
            available_key_live_dict = list(gs_context['live_dict'].keys())
        elif isinstance(gs_context['live_dict'],list):
            available_key_live_dict = gs_context['live_dict']
        selection = closest_matcher(selection,available_key_live_dict,gs_context,current_context)
        try:
            if selection in available_key_live_dict:
                temp_menu = gs_context['live_dict'][selection].copy()
                # print('8858',temp_menu)
                try:
                    option_to_user = list(temp_menu.keys())#available dictionary
                    response['title'] = temp_menu['prompt']
                except:
                    option_to_user = temp_menu
                    response['title'] = 'Please select the options'
                gs_context['menu_handled'] = True
                option_to_user = [x for x in option_to_user if x not in ['Invalid-choice-message','prompt']]
                # response['title'] = temp_menu['prompt']
                if 'Category' not in option_to_user:
                    response['options'] = option_to_user
                    gs_context['live_dict'] = temp_menu
                    # print('options without Category')
                    gs_context['active_menu'] = selection
                    gs_context['active_menu_state'].append(selection)
                    # print('normal',response)
                else:
                    category_options = temp_menu['Category']
                    print('options with Category')
                    other_option = temp_menu.keys()
                    left_options = [x for x in other_option if x not in ['Category']]
                    temp_temp_menu = {}
                    for key,value in temp_menu.items():
                        if key in left_options:
                            temp_temp_menu[key] = value
                    for keys in category_options:
                        temp_temp_menu[keys] = ''
                    # print(temp_temp_menu)
                    gs_context['live_dict'] = temp_temp_menu
                    gs_context['active_menu'] = selection
                    gs_context['active_menu_state'].append(selection)
                    left_options.extend(category_options)
                    response['options'] = left_options
                    # print(left_options)
            else:
                temp_menu = gs_context['live_dict'].copy()
                print('Wrong option else')
                response['title'] = gs_context['live_dict']['Invalid-choice-message']
                option_to_user = gs_context['live_dict'].keys()
                option_to_user = [x for x in option_to_user if x not in ['Invalid-choice-message','prompt']]
                response['options'] = option_to_user
            if temp_menu['card_type_{}'.format(channel)] == 'list':
                try:
                    response['option_desc'] = gs_context['live_dict']['Value_Description']
                except:
                    response['option_desc'] = 'None' ## Also handle this in the format_response_card_list
                response_card_distributor(gs_context,current_context,response,temp_menu)
            else:
                try:
                    response['url'] = gs_context['live_dict']['url']
                    print('This is here 4598')
                except:
                    response['url'] = 'None' ## Also change the card type to text
                response_card_distributor(gs_context,current_context,response,temp_menu)
        except Exception as e:
            if selection in gs_context['live_dict']:
                print('Here I am in the except command')
                gs_context['active_menu_state'].append(selection)
                gs_context['menu_handled']=False
                current_context["current_intent"]={'intent': 'Menu_Completed', 'entities': {}, 'user_text': ''}
            else:
                print(e, 'Thas is a wrong option please choose the one from the below')
                response['title'] = 'Thas is a wrong option please choose the one from the below'
                option_to_user = gs_context['live_dict']
                gs_context['menu_handled']=True
                response['options'] = option_to_user
                temp_menu = option_to_user
                if temp_menu['card_type_{}'.format(channel)] == 'list':
                    try:
                        response['option_desc'] = gs_context['live_dict']['Value_Description']
                    except:
                        response['option_desc'] = 'None' ## Also handle this in the format_response_card_list
                    response_card_distributor(gs_context,current_context,response,temp_menu)
                else:
                    try:
                        response['url'] = gs_context['live_dict']['url']
                    except:
                        response['url'] = 'None'
                        print('This is here 4598_2')
                    response_card_distributor(gs_context,current_context,response,temp_menu)
    #             print(option_to_user)
                return
        return


from json import dumps
from rasa_sdk.events import (SlotSet)


def post_handle(dispatcher, gs_context, current_context):
    response_queue = current_context["response_queue"]
    for element in response_queue:
        if type(element) == (str):
            dispatcher.utter_message(text=element)
        else:
            dispatcher.utter_message(text=dumps(element))
    return


def get_events(gs_context, current_context):

    return SlotSet('gs_context', dumps(gs_context))



global_context = {}

global_context['mongo_connection'] = ""
global_context['i18n_required'] = False

def set_mongo_connection_object(mongo_connection):
    global global_context
    global_context['mongo_connection'] = mongo_connection

def set_i18n_required_flag(i18n_required):
    global global_context
    global_context['i18n_required'] = i18n_required



from . import gs_global
## Get connection object from global_context


## List of all the keys that are present in response_details & should also be translated
## In case any text to be translated from response_details dict is present in nested lists & dicts, translatable_keys must contain all keys needed to traverse the path until it reaches the text to be translated
whatsapp_translatable_keys_text = ['message']
whatsapp_translatable_keys_quickreply = ['header', 'body', 'caption', 'options']
whatsapp_translatable_keys_list = ['main_title', 'main_body', 'button_text', 'sections', 'title', 'options', 'description', 'postbackText']
gip_translatable_keys_text = ['message']
gip_translatable_keys_quickreply = ['text', 'options','title']
gip_translatable_keys_list = ['globalbuttons', 'title', 'items', 'subtitle', 'options']
gip_translatable_keys_generic_card = ['message','title_0','action0_content','action0_to','title_1','action1_content','action1_to','title_2','action2_content','action2_to','title_3','action3_content','action3_to','title_4','action4_content','action4_to''title_5','action5_content','action5_to','title_6','action6_content','action6_to','title_7','action7_content','action7_to','title_8','action8_content','action8_to','title_9','action9_content','action9_to']
gip_translatable_keys_catalogue = ['items','title','subtitle','options','buttons']
insta_translatable_keys_quickreply = ['text', 'options']
insta_translatable_keys_catalogue = ['items','title','subtitle','options']
insta_translatable_keys_text = ['message']

translatable_keys_for_channel = {
    'whatsapp':{'text':whatsapp_translatable_keys_text, 'quick_reply':whatsapp_translatable_keys_quickreply,'list':whatsapp_translatable_keys_list},
    'gip':{'text':gip_translatable_keys_text, 'quick_reply':gip_translatable_keys_quickreply,'list':gip_translatable_keys_list,'generic_card':gip_translatable_keys_generic_card,'catalogue':gip_translatable_keys_catalogue},
    'instagram':{'text':insta_translatable_keys_text,'quick_reply':insta_translatable_keys_quickreply,'catalogue':insta_translatable_keys_catalogue}
    }

## default settings
translatable_keys = []
lang_code = 'en'

def get_translatable_keys(channel, card_type):
    return translatable_keys_for_channel[channel][card_type]


## Below functions to iterate through nested dicts and lists and return translated strings at the end of the heirarchy
def handle_type(obj):
    if(type(obj)==list):
        obj = handle_list(obj)
    if(type(obj)==dict):
        obj = handle_dict(obj)
    if(type(obj)==str):
        obj = handle_str(obj)
    return obj

def handle_list(list):
    temp_list=[]
    for item in list:
        item = handle_type(item)
        temp_list.append(item)
    return temp_list

def handle_dict(dict):
    for key in dict:
        if(key in translatable_keys):
            dict[key] = handle_type(dict[key])
    return dict

def handle_str(string):
    translated_text = return_if_translation_exists(string)
    return translated_text

## Get translated string from mongodb
def return_if_translation_exists(text):
    db = gs_global.global_context['mongo_connection']
    translated_text = db.get_translated_text_for_lang(text, lang_code)
    return translated_text if(translated_text is not None) else text


def i18n_translate(response_details: dict, lang_code: str, channel: str) -> dict:
    if lang_code != 'en':
        try:
            globals()['lang_code'] = lang_code
            try:
                globals()['translatable_keys'] = get_translatable_keys(channel, response_details['card_type'])
            except:
                print("Invalid card type or channel")
                return response_details
            translated_response_details = handle_type(response_details)
            return translated_response_details
        except:
            print("Translation failed")
            return response_details
    else:
        return response_details

def i18n_extract(response_details: dict):
    pass



from .gs_global import global_context,set_mongo_connection_object,set_i18n_required_flag
from .pre_handler import get_context
from .menu_handler import activate_menu
from .menu_handler import handle_active_menu
from .response_cards import show_response_card
from .post_handler import post_handle
from .post_handler import get_events
from .db_handler import db_handler
